var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { traverseLayers } from "./functions/traverse-layers";
import { settings } from "./constants/settings";
import { fastClone } from "./functions/fast-clone";
import { getLayout, hasChildren, isGroupNode } from "../lib/helpers";
const allPropertyNames = [
    "id",
    "width",
    "height",
    "currentPage",
    "cancel",
    "origin",
    "onmessage",
    "center",
    "zoom",
    "fontName",
    "name",
    "visible",
    "locked",
    "constraints",
    "relativeTransform",
    "x",
    "y",
    "rotation",
    "constrainProportions",
    "layoutAlign",
    "layoutGrow",
    "opacity",
    "blendMode",
    "isMask",
    "effects",
    "effectStyleId",
    "expanded",
    "backgrounds",
    "backgroundStyleId",
    "fills",
    "strokes",
    "strokeWeight",
    "strokeMiterLimit",
    "strokeAlign",
    "strokeCap",
    "strokeJoin",
    "dashPattern",
    "fillStyleId",
    "strokeStyleId",
    "cornerRadius",
    "cornerSmoothing",
    "topLeftRadius",
    "topRightRadius",
    "bottomLeftRadius",
    "bottomRightRadius",
    "exportSettings",
    "overflowDirection",
    "numberOfFixedChildren",
    "description",
    "layoutMode",
    "primaryAxisSizingMode",
    "counterAxisSizingMode",
    "primaryAxisAlignItems",
    "counterAxisAlignItems",
    "paddingLeft",
    "paddingRight",
    "paddingTop",
    "paddingBottom",
    "itemSpacing",
    "layoutGrids",
    "gridStyleId",
    "clipsContent",
    "guides",
    "guides",
    "selection",
    "selectedTextRange",
    "backgrounds",
    "arcData",
    "pointCount",
    "pointCount",
    "innerRadius",
    "vectorNetwork",
    "vectorPaths",
    "handleMirroring",
    "textAlignHorizontal",
    "textAlignVertical",
    "textAutoResize",
    "paragraphIndent",
    "paragraphSpacing",
    "autoRename",
    "textStyleId",
    "fontSize",
    "fontName",
    "textCase",
    "textDecoration",
    "letterSpacing",
    "lineHeight",
    "characters",
    "mainComponent",
    "scaleFactor",
    "booleanOperation",
    "expanded",
    "name",
    "type",
    "paints",
    "type",
    "fontSize",
    "textDecoration",
    "fontName",
    "letterSpacing",
    "lineHeight",
    "paragraphIndent",
    "paragraphSpacing",
    "textCase",
    "type",
    "effects",
    "type",
    "layoutGrids",
];
// The Figma nodes are hard to inspect at a glance because almost all properties are non enumerable
// getters. This removes that wrapping for easier inspecting
const cloneObject = (obj, valuesSet = new Set()) => {
    if (!obj || typeof obj !== "object") {
        return obj;
    }
    const newObj = Array.isArray(obj) ? [] : {};
    for (const property of allPropertyNames) {
        const value = obj[property];
        if (value !== undefined && typeof value !== "symbol") {
            newObj[property] = obj[property];
        }
    }
    return newObj;
};
function postSelection() {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(JSON.stringify(figma.currentPage.selection.map((obj) => cloneObject(obj)), null, 2));
        figma.ui.postMessage({
            type: "selectionChange",
            elements: fastClone(yield Promise.all(figma.currentPage.selection.map((el) => serialize(el, {
                // TODO: only need one level deep......
                withChildren: true,
            })))),
        });
    });
}
figma.on("selectionchange", () => __awaiter(void 0, void 0, void 0, function* () {
    postSelection();
}));
// This shows the HTML page in "ui.html".
figma.showUI(__html__, {
    width: settings.ui.baseWidth,
    height: settings.ui.baseHeight,
});
function processImages(layer) {
    return __awaiter(this, void 0, void 0, function* () {
        const images = getImageFills(layer);
        return (images &&
            Promise.all(images.map((image) => __awaiter(this, void 0, void 0, function* () {
                if (image === null || image === void 0 ? void 0 : image.intArr) {
                    image.imageHash = yield figma.createImage(image.intArr).hash;
                    delete image.intArr;
                }
            }))));
    });
}
function getImageFills(layer) {
    const images = Array.isArray(layer.fills) &&
        layer.fills.filter((item) => item.type === "IMAGE");
    return images;
}
const normalizeName = (str) => str.toLowerCase().replace(/[^a-z]/gi, "");
const defaultFont = { family: "Roboto", style: "Regular" };
// TODO: keep list of fonts not found
function getMatchingFont(fontStr, availableFonts) {
    return __awaiter(this, void 0, void 0, function* () {
        const familySplit = fontStr.split(/\s*,\s*/);
        for (const family of familySplit) {
            const normalized = normalizeName(family);
            for (const availableFont of availableFonts) {
                const normalizedAvailable = normalizeName(availableFont.fontName.family);
                if (normalizedAvailable === normalized) {
                    const cached = fontCache[normalizedAvailable];
                    if (cached) {
                        return cached;
                    }
                    yield figma.loadFontAsync(availableFont.fontName);
                    fontCache[fontStr] = availableFont.fontName;
                    fontCache[normalizedAvailable] = availableFont.fontName;
                    return availableFont.fontName;
                }
            }
        }
        return defaultFont;
    });
}
const fontCache = {};
function serialize(element, options = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        let fills = (element.fills && fastClone(element.fills)) || [];
        if (options.withImages && fills.length) {
            for (const fill of fills) {
                if (fill.type === "IMAGE" && fill.imageHash) {
                    const image = figma.getImageByHash(fill.imageHash);
                    try {
                        const bytes = yield image.getBytesAsync();
                        fill.intArr = bytes;
                    }
                    catch (err) {
                        console.warn("Could not get image for layer", element, fill, err);
                    }
                }
            }
        }
        // TODO: May have bg...
        const isSvg = (hasChildren(element) &&
            element.children.every((item) => item.type === "VECTOR")) ||
            element.type === "VECTOR";
        if (options.withImages &&
            // options.withVectorsExported !== false &&
            isSvg) {
            const image = yield element.exportAsync({
                // TODO: use SVG for SVGs
                format: "PNG",
                constraint: {
                    type: "SCALE",
                    value: 2,
                },
            });
            fills = [
                Object.assign({ type: "IMAGE", visible: true, scaleMode: "FIT" }, { intArr: image }),
            ];
        }
        // TODO: better way to enumerate everything, including getters, that is not function
        return Object.assign(Object.assign({}, cloneObject(element)), { fills, type: element.type === "VECTOR" ? "RECTANGLE" : element.type, data: JSON.parse(element.getSharedPluginData("builder", "data") || "{}"), children: (options.withChildren &&
                element.children &&
                !isSvg &&
                (yield Promise.all(element.children
                    .filter((child) => child.visible)
                    .map((child) => serialize(child, options))))) ||
                undefined });
    });
}
function assign(a, b) {
    for (const key in b) {
        const value = b[key];
        if (key === "data" && value && typeof value === "object") {
            const currentData = JSON.parse(a.getSharedPluginData("builder", "data") || "{}") || {};
            const newData = value;
            const mergedData = Object.assign({}, currentData, newData);
            // TODO merge plugin data
            a.setSharedPluginData("builder", "data", JSON.stringify(mergedData));
        }
        else if (typeof value != "undefined" &&
            ["width", "height", "type", "ref", "children", "svg"].indexOf(key) === -1) {
            try {
                a[key] = b[key];
            }
            catch (err) {
                console.warn(`Assign error for property "${key}"`, a, b, err);
            }
        }
    }
}
const isImportErrorsKey = "isImportErrors";
function clearAllErrors() {
    figma.currentPage.children.forEach((el) => {
        if (el.getPluginData(isImportErrorsKey) === "true") {
            el.remove();
        }
    });
}
const importableLayerTypes = new Set([
    "RECTANGLE",
    "FRAME",
    "TEXT",
    "COMPONENT",
    "LINE",
    "INSTANCE",
]);
const isNotImportable = (node) => 
// Don't show warnings for invisble nodes, we don't import them
!node.visible
    ? false
    : (node.children &&
        getLayout(node) === "unknown") ||
        !importableLayerTypes.has(node.type);
const getAbsolutePositionRelativeToArtboard = (node) => {
    var _a;
    if (typeof node.x !== "number" ||
        !node.parent ||
        ["PAGE", "DOCUMENT"].includes(node.type)) {
        return { x: 0, y: 0 };
    }
    const position = {
        x: node.x,
        y: node.y,
    };
    if (["PAGE", "DOCUMENT"].includes(node.parent.type)) {
        return position;
    }
    let parent = node;
    while ((parent = parent.parent)) {
        if (!isGroupNode(parent) && typeof parent.x === "number") {
            position.x += parent.x;
            position.y += parent.y;
        }
        // This is the end
        if (["PAGE", "DOCUMENT"].includes((_a = parent.parent) === null || _a === void 0 ? void 0 : _a.type)) {
            break;
        }
    }
    return position;
};
const getAbsolutePositionRelativeToRootLayer = (node, rootPosition) => {
    const nodeAbsolutePosition = getAbsolutePositionRelativeToArtboard(node);
    return {
        x: nodeAbsolutePosition.x - rootPosition.x,
        y: nodeAbsolutePosition.y - rootPosition.y,
    };
};
const hasInvisibleParent = (node) => {
    let parent = node;
    do {
        if (parent.visible === false) {
            return true;
        }
    } while ((parent = parent.parent));
    return false;
};
// Returns true if valid
function checkIfCanGetCode() {
    return __awaiter(this, void 0, void 0, function* () {
        clearAllErrors();
        const selected = figma.currentPage.selection[0];
        if (!selected) {
            return false;
        }
        const invalidLayers = [];
        yield traverseLayers(selected, (node) => {
            if (!hasInvisibleParent(node) && isNotImportable(node)) {
                invalidLayers.push(node);
            }
        });
        if (invalidLayers.length) {
            const errorFrame = figma.createFrame();
            errorFrame.name = "Export to code errors - delete me anytime";
            const absolutePosition = getAbsolutePositionRelativeToArtboard(selected);
            errorFrame.x = absolutePosition.x;
            errorFrame.y = absolutePosition.y;
            errorFrame.fills = [];
            errorFrame.resize(selected.width || 1, selected.height || 1);
            errorFrame.setPluginData(isImportErrorsKey, "true");
            for (const invalidLayer of invalidLayers) {
                const errorLayer = figma.createRectangle();
                errorLayer.setPluginData(isImportErrorsKey, "true");
                if (invalidLayer.type === "VECTOR") {
                    errorLayer.name = `"${invalidLayer.name}" needs to be a rasterized image`;
                }
                else {
                    errorLayer.name = `"${invalidLayer.name}" needs to use autolayout`;
                }
                const { x, y } = getAbsolutePositionRelativeToRootLayer(invalidLayer, absolutePosition);
                errorLayer.x = x;
                errorLayer.y = y;
                errorLayer.fills = [];
                errorLayer.resize(invalidLayer.width || 1, invalidLayer.height || 1);
                errorLayer.strokeWeight = 4;
                errorLayer.strokes = [
                    {
                        type: "SOLID",
                        visible: true,
                        opacity: 1,
                        blendMode: "NORMAL",
                        color: {
                            r: 1,
                            g: 0,
                            b: 0,
                        },
                    },
                ];
                errorFrame.appendChild(errorLayer);
            }
        }
        return !invalidLayers.length;
    });
}
// Calls to "parent.postMessage" from within the HTML page will trigger this
// callback. The callback will be passed the "pluginMessage" property of the
// posted message.
figma.ui.onmessage = (msg) => __awaiter(void 0, void 0, void 0, function* () {
    if (msg.type === "resize") {
        figma.ui.resize(msg.width, msg.height);
    }
    if (msg.type === "getStorage") {
        const data = yield figma.clientStorage.getAsync("data");
        figma.ui.postMessage({
            type: "storage",
            data,
        });
    }
    if (msg.type === "init") {
        postSelection();
    }
    if (msg.type === "setStorage") {
        const data = msg.data;
        figma.clientStorage.setAsync("data", data);
    }
    if (msg.type === "checkIfCanGetCode") {
        const canGet = yield checkIfCanGetCode();
        figma.ui.postMessage({
            type: "canGetCode",
            value: canGet,
        });
    }
    if (msg.type === "getSelectionWithImages") {
        figma.ui.postMessage({
            type: "selectionWithImages",
            elements: yield Promise.all(figma.currentPage.selection.map((el) => serialize(el, {
                withChildren: true,
                withImages: true,
            }))),
        });
    }
    if (msg.type === "updateElements") {
        const elements = msg.elements;
        for (const element of elements) {
            const el = figma.getNodeById(element.id);
            if (el) {
                assign(el, element);
            }
        }
    }
    if (msg.type === "clearErrors") {
        clearAllErrors();
    }
    if (msg.type === "import") {
        const availableFonts = (yield figma.listAvailableFontsAsync()).filter((font) => font.fontName.style === "Regular");
        yield figma.loadFontAsync(defaultFont);
        const { data } = msg;
        const { layers } = data;
        const rects = [];
        let baseFrame = figma.currentPage;
        // TS bug? TS is implying that frameRoot is PageNode and ignoring the type declaration
        // and the reassignment unless I force it to treat baseFrame as any
        let frameRoot = baseFrame;
        for (const rootLayer of layers) {
            yield traverseLayers(rootLayer, (layer, parent) => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    if (layer.type === "FRAME" || layer.type === "GROUP") {
                        const frame = figma.createFrame();
                        frame.x = layer.x;
                        frame.y = layer.y;
                        frame.resize(layer.width || 1, layer.height || 1);
                        assign(frame, layer);
                        rects.push(frame);
                        ((parent && parent.ref) || baseFrame).appendChild(frame);
                        layer.ref = frame;
                        if (!parent) {
                            frameRoot = frame;
                            baseFrame = frame;
                        }
                        // baseFrame = frame;
                    }
                    else if (layer.type === "SVG") {
                        const node = figma.createNodeFromSvg(layer.svg);
                        node.x = layer.x;
                        node.y = layer.y;
                        node.resize(layer.width || 1, layer.height || 1);
                        layer.ref = node;
                        rects.push(node);
                        assign(node, layer);
                        ((parent && parent.ref) || baseFrame).appendChild(node);
                    }
                    else if (layer.type === "RECTANGLE") {
                        const rect = figma.createRectangle();
                        if (getImageFills(layer)) {
                            yield processImages(layer);
                        }
                        assign(rect, layer);
                        rect.resize(layer.width || 1, layer.height || 1);
                        rects.push(rect);
                        layer.ref = rect;
                        ((parent && parent.ref) || baseFrame).appendChild(rect);
                    }
                    else if (layer.type == "TEXT") {
                        const text = figma.createText();
                        if (layer.fontFamily) {
                            const cached = fontCache[layer.fontFamily];
                            if (cached) {
                                text.fontName = cached;
                            }
                            else {
                                const family = yield getMatchingFont(layer.fontFamily || "", availableFonts);
                                text.fontName = family;
                            }
                            delete layer.fontFamily;
                        }
                        assign(text, layer);
                        layer.ref = text;
                        text.resize(layer.width || 1, layer.height || 1);
                        text.textAutoResize = "HEIGHT";
                        const lineHeight = (layer.lineHeight && layer.lineHeight.value) || layer.height;
                        let adjustments = 0;
                        while (typeof text.fontSize === "number" &&
                            typeof layer.fontSize === "number" &&
                            (text.height > Math.max(layer.height, lineHeight) * 1.2 ||
                                text.width > layer.width * 1.2)) {
                            // Don't allow changing more than ~30%
                            if (adjustments++ > layer.fontSize * 0.3) {
                                console.warn("Too many font adjustments", text, layer);
                                // debugger
                                break;
                            }
                            try {
                                text.fontSize = text.fontSize - 1;
                            }
                            catch (err) {
                                console.warn("Error on resize text:", layer, text, err);
                            }
                        }
                        rects.push(text);
                        ((parent && parent.ref) || baseFrame).appendChild(text);
                    }
                }
                catch (err) {
                    console.warn("Error on layer:", layer, err);
                }
            }));
        }
        if (frameRoot.type === "FRAME") {
            figma.currentPage.selection = [frameRoot];
        }
        figma.ui.postMessage({
            type: "doneLoading",
            rootId: frameRoot.id,
        });
        figma.viewport.scrollAndZoomIntoView([frameRoot]);
        if (process.env.NODE_ENV !== "development") {
            figma.closePlugin();
        }
    }
    // Make sure to close the plugin when you're done. Otherwise the plugin will
    // keep running, which shows the cancel button at the bottom of the screen.
});
